/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core;

import javax.persistence.Entity;

import org.hibernate.annotations.NamedQueries;
import org.hibernate.annotations.NamedQuery;

import com.patrikdufresne.managers.ArchivableObject;

/**
 * The Product class represent one product that may be schedule.
 */
@Entity
@NamedQueries( {
        @NamedQuery(name = "listProductTableByShifts", query = "SELECT S, P, SUM(PP.number), S.team "
                + "FROM Product as P, Shift as S, ProductionEvent as PE, ProductPosition as PP "
                + "WHERE PP.product = P "
                + "AND PE.product = P "
                + "AND PE.shift = S "
                + "AND S in (:shifts) "
                + "group by S, P"),
        @NamedQuery(name = "listProductTable", query = "SELECT P, SUM(PP.number) "
                + "FROM Product as P, ProductPosition as PP "
                + "WHERE PP.product = P "
                + "AND P in (:products) "
                + "group by P") })
public class Product extends ArchivableObject {

    /**
     * Family property
     */
    public static final String FAMILY = "family"; //$NON-NLS-1$

    /**
     * Name property
     */
    public static final String NAME = "name"; //$NON-NLS-1$
    /**
     * RefId property
     */
    public static final String REFID = "refId"; //$NON-NLS-1$
    private static final long serialVersionUID = 1833794757829504482L;

    /**
     * Define the product family.
     */
    private String family;

    /**
     * Define the product's name.
     * 
     */
    private String name;

    /**
     * Define a reference number to the product.
     * 
     */
    private String refId;

    public String getFamily() {
        return this.family;
    }

    /**
     * Getter of the property <tt>name</tt>
     * 
     * @return Returns the name.
     */
    public String getName() {
        return this.name;
    }

    /**
     * Getter of the property <tt>refId</tt>
     * 
     * @return Returns the refId.
     */
    public String getRefId() {
        return this.refId;
    }

    public void setFamily(String family) {
        this.changeSupport.firePropertyChange(FAMILY, this.family, this.family = family);
    }

    /**
     * Setter of the property <tt>name</tt>
     * 
     * @param name
     *            The name to set.
     */
    public void setName(String name) {
        this.changeSupport.firePropertyChange(NAME, this.name, this.name = name);
    }

    /**
     * Setter of the property <tt>refId</tt>
     * 
     * @param refId
     *            The refId to set.
     */
    public void setRefId(String refId) {
        this.changeSupport.firePropertyChange(REFID, this.refId, this.refId = refId);
    }

    @Override
    public String toString() {
        return "Product [id=" + this.id + ", name=" + this.name + ", refId=" //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                + this.refId
                + "]"; //$NON-NLS-1$
    }

}
