/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.ManyToOne;

import com.patrikdufresne.managers.ManagedObject;

/**
 * This object represent the relation between products and position.
 * 
 * @author Patrik Dufresne
 * 
 */
@Entity
public class ProductPosition extends ManagedObject {
    private static final long serialVersionUID = 1396115605522547622L;
    /**
     * Number property
     */
    public static final String NUMBER = "number"; //$NON-NLS-1$
    /**
     * Position property
     */
    public static final String POSITION = "position"; //$NON-NLS-1$
    /**
     * Product property
     */
    public static final String PRODUCT = "product"; //$NON-NLS-1$

    /**
     * Define the number of the given position required to produce the attached
     * product.
     * 
     * @uml.property name="number"
     */
    private Integer number;

    /**
     * Define the position required to be fill to produce the product.
     * 
     * @uml.property name="position"
     * @uml.associationEnd inverse="productPosition:com.planimou.core.Position"
     */
    private Position position;

    /**
     * The product to be produce.
     * 
     * @uml.property name="product"
     * @uml.associationEnd inverse="productPosition:com.planimou.core.Product"
     */
    private Product product;

    /**
     * Getter of the property <tt>number</tt>
     * 
     * @return Returns the number.
     * @uml.property name="number"
     */
    @Column(nullable = false)
    public Integer getNumber() {
        return this.number;
    }

    /**
     * Getter of the property <tt>position</tt>
     * 
     * @return Returns the position.
     * @uml.property name="position"
     */
    @ManyToOne(optional = false)
    public Position getPosition() {
        return this.position;
    }

    /**
     * Getter of the property <tt>product</tt>
     * 
     * @return Returns the product.
     * @uml.property name="product"
     */
    @ManyToOne(optional = false)
    public Product getProduct() {
        return this.product;
    }

    /**
     * Setter of the property <tt>number</tt>
     * 
     * @param number
     *            The number to set.
     * @uml.property name="number"
     */
    public void setNumber(Integer number) {
        this.changeSupport.firePropertyChange(NUMBER, this.number, this.number = number);
    }

    /**
     * Setter of the property <tt>position</tt>
     * 
     * @param position
     *            The position to set.
     * @uml.property name="position"
     */
    public void setPosition(Position position) {
        this.changeSupport.firePropertyChange(POSITION, this.position, this.position = position);
    }

    /**
     * Setter of the property <tt>product</tt>
     * 
     * @param product
     *            The product to set.
     * @uml.property name="product"
     */
    public void setProduct(Product product) {
        this.changeSupport.firePropertyChange(PRODUCT, this.product, this.product = product);
    }

    @Override
    public String toString() {
        return "ProductPosition [number=" + this.number //$NON-NLS-1$
                + ", position=" + this.position //$NON-NLS-1$ 
                + ", product=" + this.product + "]"; //$NON-NLS-1$ //$NON-NLS-2$
    }

}
