/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core;

import javax.persistence.Entity;
import javax.persistence.ManyToOne;

import org.hibernate.annotations.NamedQueries;
import org.hibernate.annotations.NamedQuery;

import com.patrikdufresne.managers.ManagedObject;

/**
 * A qualification represent a position known by an employee.
 * 
 * @author Patrik Dufresne
 * 
 */
@Entity
@NamedQueries( { @NamedQuery(name = "getQualificationTableByPositions", query = "SELECT E, Po, Se "
        + "FROM Qualification Q JOIN Q.employee as E JOIN Q.position as Po JOIN Po.section as Se "
        + "WHERE Q.position in :positions AND E.archivedDate is null") })
public class Qualification extends ManagedObject {

    /**
     * Employee property.
     */
    public static final String EMPLOYEE = "employee"; //$NON-NLS-1$
    /**
     * Position property
     */
    public static final String POSITION = "position"; //$NON-NLS-1$
    private static final long serialVersionUID = -7961409326939904862L;

    private Employee employee;

    private Position position;

    /**
     * Getter of the property <tt>employee</tt>
     * 
     * @return Returns the employee.
     */
    @ManyToOne(optional = false)
    public Employee getEmployee() {
        return this.employee;
    }

    /**
     * Getter of the property <tt>position</tt>
     * 
     * @return Returns the position.
     */
    @ManyToOne(optional = false)
    public Position getPosition() {
        return this.position;
    }

    /**
     * Setter of the property <tt>employee</tt>
     * 
     * @param employee
     *            The employee to set.
     */
    public void setEmployee(Employee employee) {
        this.changeSupport.firePropertyChange(EMPLOYEE, this.employee, this.employee = employee);
    }

    /**
     * Setter of the property <tt>position</tt>
     * 
     * @param position
     *            The position to set.
     */
    public void setPosition(Position position) {
        this.changeSupport.firePropertyChange(POSITION, this.position, this.position = position);
    }

    @Override
    public String toString() {
        return "Qualification [position=" + this.position //$NON-NLS-1$
                + ", employee=" + this.employee + "]"; //$NON-NLS-1$ //$NON-NLS-2$
    }

}
