/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.comparators;

import java.util.Comparator;

import org.apache.commons.collections.ComparatorUtils;
import org.apache.commons.collections.Transformer;
import org.apache.commons.collections.comparators.ComparatorChain;

import com.planimod.core.AbstractCalendarEvent;
import com.planimod.core.NonAvailability;

public class NonAvailabilityComparators {

    /**
     * Private constructor for utility class.
     */
    private NonAvailabilityComparators() {

    }

    /**
     * Create a new comparator.
     * 
     * @return
     */
    public static Comparator<NonAvailability> byDateAndEmployeeName() {

        ComparatorChain chain = new ComparatorChain();

        chain.addComparator(ComparatorUtils.nullHighComparator(ComparatorUtils.transformedComparator(ComparatorUtils.nullLowComparator(ComparatorUtils
                .naturalComparator()), new Transformer() {
            @Override
            public Object transform(Object arg0) {
                return ((AbstractCalendarEvent) arg0).getStartDate();
            }
        })));

        chain.addComparator(ComparatorUtils.nullHighComparator(ComparatorUtils.transformedComparator(ComparatorUtils.nullLowComparator(EmployeeComparators
                .byName()), new Transformer() {
            @Override
            public Object transform(Object arg0) {
                return ((NonAvailability) arg0).getEmployee();
            }
        })));

        return chain;
    }

    /**
     * Compare Non-Availability by employee's name.
     * 
     * @return
     */
    @SuppressWarnings("unchecked")
    public static Comparator<? super NonAvailability> byEmployeeName() {

        return ComparatorUtils.nullHighComparator(ComparatorUtils.transformedComparator(EmployeeComparators.byName(), new Transformer() {
            @Override
            public Object transform(Object input) {
                return ((NonAvailability) input).getEmployee();
            }
        }));

    }
}
