/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.comparators;

import java.util.Comparator;

import org.apache.commons.collections.ComparatorUtils;
import org.apache.commons.collections.Transformer;
import org.apache.commons.collections.comparators.ComparatorChain;

import com.planimod.core.Position;

/**
 * Utility class to sort position.
 * 
 * @author Patrik Dufresne
 * 
 */
public class PositionComparators {
    /**
     * Comparator to position bye name.
     * 
     * @return
     */
    public static Comparator<Position> byName() {
        return ComparatorUtils.nullLowComparator(ComparatorUtils.transformedComparator(
                ComparatorUtils.nullLowComparator(ComparatorUtils.naturalComparator()),
                new Transformer() {
                    @Override
                    public Object transform(Object arg0) {
                        return ((Position) arg0).getName();
                    }
                }));
    }

    public static Comparator<Position> bySectionClassifiedAndName() {

        ComparatorChain chain = new ComparatorChain();

        // Compare section's name
        chain.addComparator(ComparatorUtils.nullLowComparator(ComparatorUtils.transformedComparator(SectionComparators.byName(), new Transformer() {
            @Override
            public Object transform(Object arg0) {
                return ((Position) arg0).getSection();
            }
        })));

        // Classified & name
        chain.addComparator(byClassifiedName());

        return chain;
    }

    /**
     * Comparator to sort Position by classified position (first) and then by
     * name.
     * 
     * @return
     */
    public static Comparator<Position> byClassifiedName() {
        ComparatorChain chain = new ComparatorChain();

        // Classified
        chain.addComparator(ComparatorUtils.nullLowComparator(ComparatorUtils.transformedComparator(ComparatorUtils.booleanComparator(true), new Transformer() {
            @Override
            public Object transform(Object arg0) {
                return Boolean.valueOf(((Position) arg0).getClassified());
            }
        })));

        // Name
        chain.addComparator(byName());

        return chain;
    }
}
