/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.comparators;

import java.util.Comparator;

import org.apache.commons.collections.ComparatorUtils;
import org.apache.commons.collections.Transformer;

import com.planimod.core.AbstractCalendarEvent;
import com.planimod.core.ProductionEvent;

public class ProductionEventComparators {

    @SuppressWarnings("unchecked")
    public static Comparator<AbstractCalendarEvent> byDate() {
        return ComparatorUtils.transformedComparator(ComparatorUtils.nullLowComparator(ComparatorUtils.naturalComparator()), new Transformer() {
            @Override
            public Object transform(Object arg0) {
                return ((ProductionEvent) arg0).getShift().getStartDate();
            }
        });
    }

    @SuppressWarnings("unchecked")
    public static Comparator<? super ProductionEvent> byProductRefId() {
        return ComparatorUtils.transformedComparator(ComparatorUtils.nullLowComparator(new HumanNaturalComparator()), new Transformer() {
            @Override
            public Object transform(Object arg0) {
                return ((ProductionEvent) arg0).getProduct().getRefId();
            }
        });
    }

    /**
     * Compare Production event using the familly.
     * 
     * @return
     */
    @SuppressWarnings("unchecked")
    public static Comparator<? super ProductionEvent> byRefIdFamilyName() {
        return ComparatorUtils.nullHighComparator(ComparatorUtils.transformedComparator(ProductComparators.byRefIdFamilyName(), new Transformer() {
            @Override
            public Object transform(Object arg0) {
                return ((ProductionEvent) arg0).getProduct();
            }
        }));
    }

}
