/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.comparators;

import java.util.Comparator;

import org.apache.commons.collections.ComparatorUtils;
import org.apache.commons.collections.Transformer;
import org.apache.commons.collections.comparators.ComparatorChain;

import com.planimod.core.Task;

/**
 * Utility class used to provide comparators for {@link Task} class.
 * 
 * @author Patrik Dufresne
 * 
 */
public class TaskComparators {

    /**
     * Compare {@link Task} by team, by date and Section
     * 
     * @return the comparator
     */
    public static Comparator<Task> byTeam() {
        ComparatorChain chain = new ComparatorChain();

        chain.addComparator(ComparatorUtils.nullHighComparator(ComparatorUtils.transformedComparator(TeamComparators.byName(), new Transformer() {
            @Override
            public Object transform(Object input) {
                return ((Task) input).getProductionEvent().getShift().getTeam();
            }
        })));

        chain.addComparator(ComparatorUtils.nullHighComparator(ComparatorUtils.transformedComparator(TimeRangeComparators.byDate(), new Transformer() {
            @Override
            public Object transform(Object input) {
                return ((Task) input).getProductionEvent().getShift();
            }
        })));

        chain.addComparator(ComparatorUtils.nullHighComparator(ComparatorUtils.transformedComparator(SectionComparators.byName(), new Transformer() {
            @Override
            public Object transform(Object input) {
                return ((Task) input).getPosition().getSection();
            }
        })));

        return chain;
    }

}
