/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import com.patrikdufresne.ilp.Constraint;
import com.patrikdufresne.ilp.util.ConstraintSnapshot;
import com.patrikdufresne.ilp.util.Constraints;

/**
 * Abstract implementation to support a single constraint.
 */
public abstract class AbstractConstraintRunnable extends SolveRunnable {

    protected Constraint constraint;

    protected ConstraintSnapshot constraintSnapshot;

    /**
     * @param problem
     *            reference to ProblemBuilder
     */
    public AbstractConstraintRunnable(ProblemBuilder problem) {
        super(problem);
    }

    /**
     * This implementation restore the constraint snapshot.
     */
    @Override
    public void engage() {
        if (this.constraintSnapshot == null) {
            throw new RuntimeException("snaptshot not available."); //$NON-NLS-1$
        }
        this.constraint = this.constraintSnapshot.restore(pb.lp);
        this.constraintSnapshot = null;
    }

    /**
     * This implementation release the constraints and keep a reference of it as a snapshot.
     */
    @Override
    public void release() {
        if (this.constraint == null) {
            throw new RuntimeException("constraint not available.");//$NON-NLS-1$
        }
        this.constraintSnapshot = Constraints.release(this.constraint);
        this.constraint = null;
    }

    /**
     * This implementation check if the constraint already exists.
     */
    @Override
    public boolean run() throws GeneratePlanifException {
        // Check if the constraint already exists.
        if (this.constraint != null) {
            throw new RuntimeException("constraint already exists"); //$NON-NLS-1$
        }
        return super.run();
    }

    /**
     * This function release and dispose the constraint.
     */
    @Override
    protected void unfeasible() throws GeneratePlanifException {
        // Dispose the constraint
        if (this.constraint != null) {
            this.constraint.dispose();
            this.constraint = null;
        }
    }
}