/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import com.planimod.core.Employee;
import com.planimod.core.Team;

/**
 * Used to create a warning when an employee is planned to work on a classified position that is not in his preferences
 * and an employee with less seniority is planned to work on this employee prefered team.
 * 
 * @author Karl St-Cyr
 * 
 */
public class HasSeniorityToWorkOnPreferedTeamWarning extends ProposedWarning {

    protected Employee bumpEmployee;

    protected Team bumpTeam;

    /**
     * 
     * @param employee
     *            the employee who is the target of the warning.
     * @param bumpEmployee
     *            the employee who is planned to work on a position that could be assigned to the tageted employee.
     * @param bumpTeam
     *            the team on which the target could be assigned
     */
    public HasSeniorityToWorkOnPreferedTeamWarning(Employee employee, Employee bumpEmployee, Team bumpTeam) {
        super(employee);
        if (bumpEmployee == null || bumpTeam == null) {
            throw new IllegalArgumentException();
        }
        this.bumpEmployee = bumpEmployee;
        this.bumpTeam = bumpTeam;
    }

    public Employee getBumpEmployee() {
        return bumpEmployee;
    }

    public Team getBumpTeam() {
        return bumpTeam;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((bumpEmployee == null) ? 0 : bumpEmployee.hashCode());
        result = prime * result + ((bumpTeam == null) ? 0 : bumpTeam.hashCode());
        result = prime * result + ((employee == null) ? 0 : employee.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null) return false;
        if (getClass() != obj.getClass()) return false;
        HasSeniorityToWorkOnPreferedTeamWarning other = (HasSeniorityToWorkOnPreferedTeamWarning) obj;
        if (bumpEmployee == null) {
            if (other.bumpEmployee != null) return false;
        } else if (!bumpEmployee.equals(other.bumpEmployee)) return false;
        if (bumpTeam == null) {
            if (other.bumpTeam != null) return false;
        } else if (!bumpTeam.equals(other.bumpTeam)) return false;
        if (employee == null) {
            if (other.employee != null) return false;
        } else if (!employee.equals(other.employee)) return false;
        return true;
    }

}
