/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import com.planimod.core.Employee;
import com.planimod.core.Team;

/**
 * Used in cases where the generator can't assign an employee with preferred seniority to his preferred team.
 * 
 * @author Karl St-Cyr
 * 
 */
public class PreferedSeniorityCallbackReason implements ICallbackReason {

    private final Employee employee;
    private final Team team;

    /**
     * 
     * @param employee
     *            the employee with preferred seniority targeted by the callbackReason
     * @param team
     *            the preferred team of this employee
     */
    public PreferedSeniorityCallbackReason(Employee employee, Team team) {
        if (employee == null || team == null) {
            throw new IllegalArgumentException();
        }
        this.employee = employee;
        this.team = team;
    }

    public Employee getEmployee() {
        return employee;
    }

    public Team getTeam() {
        return team;
    }

    @Override
    public String toString() {
        return "PreferedSeniorityCallbackReason [employee=" + employee //$NON-NLS-1$
                + ", team=" + team //$NON-NLS-1$
                + "]"; //$NON-NLS-1$
    }

}
