/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.databinding;

import java.beans.PropertyDescriptor;

import org.eclipse.core.databinding.conversion.IConverter;
import org.eclipse.core.internal.databinding.beans.BeanPropertyHelper;

import com.planimod.core.EmployeePreference;

public class EmployeePreferredPositionValueProperty extends BeanComputedValueProperty {

    private String noneString;

    /**
     * Create a new property value.
     * 
     * @param noneString
     *            define the string value when the employee doesn't have a
     *            preferred position.
     */
    public EmployeePreferredPositionValueProperty(String noneString) {
        super(properties(), String.class);
        this.noneString = noneString;
    }

    /**
     * Create a new property value.
     */
    public EmployeePreferredPositionValueProperty() {
        this(""); //$NON-NLS-1$
    }

    /**
     * Static function used to return the list of property descriptors for this
     * computed value property.
     * 
     * @return
     */
    private static PropertyDescriptor[] properties() {
        return new PropertyDescriptor[] {
                BeanPropertyHelper.getPropertyDescriptor(EmployeePreference.class, EmployeePreference.PREFERRED_POSITION),
                BeanPropertyHelper.getPropertyDescriptor(EmployeePreference.class, EmployeePreference.PREFERRED_POSITION_TEAM) };
    }

    private IConverter converter = Converters.removeFrontNumber();

    @Override
    protected Object doGetValue(Object source) {
        EmployeePreference pref = (EmployeePreference) source;
        if (pref.getPreferredPosition() == null || pref.getPreferredPositionTeam() == null) {
            return this.noneString;
        }
        return this.converter.convert(pref.getPreferredPositionTeam().getName()) + " - " //$NON-NLS-1$
                + pref.getPreferredPosition().getName();
    }
}
