/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.databinding.state;

import org.eclipse.core.commands.IStateListener;
import org.eclipse.core.commands.State;
import org.eclipse.core.databinding.observable.Diffs;
import org.eclipse.core.databinding.property.INativePropertyListener;
import org.eclipse.core.databinding.property.ISimplePropertyListener;
import org.eclipse.core.databinding.property.NativePropertyListener;
import org.eclipse.core.databinding.property.value.SimpleValueProperty;

/**
 * Property used to observe State.
 * 
 * @author Patrik Dufresne
 * 
 */
public class StateValueProperty extends SimpleValueProperty {

    private class Listener extends NativePropertyListener implements IStateListener {

        public Listener(ISimplePropertyListener listener) {
            super(StateValueProperty.this, listener);
        }

        @Override
        protected void doAddTo(Object source) {
            ((State) source).addListener(this);
        }

        @Override
        protected void doRemoveFrom(Object source) {
            ((State) source).removeListener(this);
        }

        @Override
        public void handleStateChange(State state, Object oldValue) {
            fireChange(state, Diffs.createValueDiff(oldValue, state.getValue()));
        }

    }

    private Object type;

    /**
     * Create an untyped value property.
     */
    public StateValueProperty() {
        this(null);
    }

    public StateValueProperty(Object type) {
        this.type = type;
    }

    @Override
    public INativePropertyListener adaptListener(ISimplePropertyListener listener) {
        return new Listener(listener);
    }

    @Override
    protected Object doGetValue(Object source) {
        return ((State) source).getValue();
    }

    @Override
    protected void doSetValue(Object source, Object value) {
        ((State) source).setValue(value);
    }

    @Override
    public Object getValueType() {
        return this.type;
    }

}
