/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.views.admin.product;

import java.util.Collection;
import java.util.Collections;

import org.eclipse.core.databinding.observable.IObservable;
import org.eclipse.core.databinding.observable.value.IObservableValue;

import com.patrikdufresne.managers.ManagerException;
import com.patrikdufresne.managers.databinding.ManagedObjectComputedSet;
import com.planimod.core.PlanimodManagers;
import com.planimod.core.Product;
import com.planimod.core.ProductPosition;

public class ProductPositionByProductComputedSet extends ManagedObjectComputedSet {

    protected IObservableValue product;

    public ProductPositionByProductComputedSet(PlanimodManagers managers, IObservableValue product) {
        super(managers, ProductPosition.class, new IObservable[] { product });
        this.product = product;
    }

    @Override
    protected Collection doList() throws ManagerException {
        if (getProduct() == null) {
            return Collections.EMPTY_SET;
        }
        return ((PlanimodManagers) getManagers()).getProductPositionManager().listByProduct(getProduct());
    }

    @Override
    protected boolean doSelect(Object element) {
        if (getProduct() == null) {
            return false;
        }
        return element instanceof ProductPosition && ((ProductPosition) element).getProduct().equals(getProduct());
    }

    protected Product getProduct() {
        if (this.product.getValue() instanceof Product) {
            return (Product) this.product.getValue();
        }
        return null;
    }

}
