/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.views.planif;

import java.beans.PropertyDescriptor;

import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.internal.databinding.beans.BeanPropertyHelper;

import com.planimod.core.Employee;
import com.planimod.core.Task;
import com.planimod.ui.databinding.BeanComputedValueProperty;

/**
 * This property is used to locked an employee to a specified task.
 * 
 * @author Patrik Dufresne
 * 
 */
public class TaskLockedValueProperty extends BeanComputedValueProperty {

    /**
     * Static function used to return the list of property descriptors for this
     * computed value property.
     * 
     * @return
     */
    private static PropertyDescriptor[] properties() {
        return new PropertyDescriptor[] {
                BeanPropertyHelper.getPropertyDescriptor(Task.class, Task.EMPLOYEE),
                BeanPropertyHelper.getPropertyDescriptor(Task.class, Task.LOCKED) };
    }

    IObservableValue employee;

    /**
     * Create a new intance of this class with the employee value specified.
     * 
     * @param employee
     *            the employee value to be used when locking the task.
     */
    public TaskLockedValueProperty(IObservableValue employee) {
        super(properties(), Boolean.TYPE);
        if (employee == null) {
            throw new NullPointerException();
        }
        this.employee = employee;
    }

    @Override
    protected Object doGetValue(Object source) {
        return Boolean.valueOf(((Task) source).getLocked());
    }

    @Override
    protected void doSetValue(Object source, Object value) {
        if (value instanceof Boolean) {
            ((Task) source).setLocked(((Boolean) value).booleanValue());
            Employee emp = getEmployee();
            if (emp != null) {
                ((Task) source).setEmployee(emp);
            }
        }
    }

    /**
     * Return the current employee value or null
     * 
     * @return
     */
    protected Employee getEmployee() {
        if (this.employee.getValue() instanceof Employee) {
            return (Employee) this.employee.getValue();
        }
        return null;
    }
}
