/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package com.planimod.ui.views.planif.printing;

import static com.patrikdufresne.util.Localized._;

import java.util.Date;

import org.eclipse.core.databinding.observable.value.WritableValue;
import org.eclipse.core.databinding.validation.MultiValidator;
import org.eclipse.core.databinding.validation.ValidationStatus;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.databinding.swt.WidgetProperties;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DateTime;

import com.patrikdufresne.jface.wizard.AbstractWizardPage;
import com.patrikdufresne.util.Localized;
import com.planimod.core.PlanimodManagers;
import com.planimod.ui.databinding.ObservableWeekEnd;
import com.planimod.ui.databinding.ObservableWeekStart;
import com.planimod.ui.views.planif.TaskComputedSet;

/**
 * This class is used to create a planif transfert: does a delta between two weeks.
 * 
 * @author Patrik Dufresne
 * 
 */
public class PlanifWeekTransfertWizardPage extends AbstractWizardPage {

    public static final String PAGE_NAME = "PlanifWeekTransfertWizardPage"; //$NON-NLS-1$
    private DateTime dateTime;

    /**
     * Create a new page
     */
    protected PlanifWeekTransfertWizardPage() {
        super(PAGE_NAME);
        setTitle(_(PlanifWeekTransfertWizardPage.class, "PlanifWeekTransfertWizardPage.title", 0));
        setPageComplete(false);
    }

    @Override
    protected void bindValues() {
        final WritableValue previousDate = ((PlanifWeekTransfertPrintWizard) getWizard()).previousDate;
        final PlanimodManagers managers = ((PlanifWeekTransfertPrintWizard) getWizard()).managers;
        final ObservableWeekStart start = new ObservableWeekStart(managers, ((PlanifWeekTransfertPrintWizard) getWizard()).date);
        final TaskComputedSet tasks = new TaskComputedSet(managers, new ObservableWeekStart(managers, previousDate), new ObservableWeekEnd(
                managers,
                previousDate));

        getDbc().bindValue(WidgetProperties.selection().observe(this.dateTime), previousDate);

        /*
         * Add validation
         */
        getDbc().addValidationStatusProvider(new MultiValidator() {

            @Override
            protected IStatus validate() {

                /*
                 * Validates that both entries are dates
                 */
                if (!(start.getValue() instanceof Date) || !(previousDate.getValue() instanceof Date)) {
                    throw new IllegalStateException();
                }

                /*
                 * Validates that the selected date is before the currently used date
                 */
                if (((Date) start.getValue()).compareTo((Date) previousDate.getValue()) <= 0) {
                    return ValidationStatus.error(Localized.format(PlanifWeekTransfertWizardPage.class, "PlanifWeekTransfertWizardPage.dateNotPrevious", 0));
                }

                /*
                 * Validates that the selected date is not empty
                 */
                if (tasks.size() == 0) {
                    return ValidationStatus.error(Localized.format(PlanifWeekTransfertWizardPage.class, "PlanifWeekTransfertWizardPage.noTaskInPrevious", 0));
                }
                return ValidationStatus.OK_STATUS;
            }
        });
    }

    /**
     * This implementation create the required widgets to setup the printing.
     * 
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void createControl(Composite parent) {
        Composite comp = new Composite(parent, SWT.NONE);
        comp.setLayout(GridLayoutFactory.fillDefaults().create());
        this.dateTime = new DateTime(comp, SWT.CALENDAR);
        this.dateTime.setLayoutData(GridDataFactory.fillDefaults().align(SWT.CENTER, SWT.CENTER).grab(true, true).create());
        setControl(comp);

        bind();
    }
}
